/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef PHATUNINGCONTROLLER_H
#define PHATUNINGCONTROLLER_H

#include <Controller.h>
#include "CAENPhaController.h"

namespace npp {

/*!
 * \class CAENVPhaDriver
 * \brief Main class for the CAENPhaDriver device driver
 *
 * \par
 * Class for CAENPhaDriver driver. Implements DeviceDriver methods.
 * \par
 * The CAENPhaDriver device manages the global functionality of the
 * CAEN V1724 and CAEN V1730 VME Digitizers.
 */


class PhaTuningController: public ExperimentController , public controller::Stoppable {

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	PhaTuningController(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~PhaTuningController();

	/**
	 * \brief Postconfiguration
	 */
	virtual void postConfiguration();

	// Channels properties
	Property<float64> signalRiseTime;
	Property<float64> signalDecayTime;
	Property<float64> trapezoidRiseTime;
	Property<float64> trapezoidFlatTop;
	Property<int32> triggerSmoothing;
	Property<int32> baselineMean;
	Property<int32> peakMean;
	Property<int32> displayChannel;
	Property<int32> nbControllers;
	Property<int32> activeController;

	DynamicProperty<int32> controllerNumberList;
	DynamicProperty<std::string> controllerNameList;

protected:

	/*!
	 * \brief Start command
	 */
	virtual void start();

	/*!
	 * \brief Stop command
	 */
	virtual void stop();

	/*!
	 * \brief Update progress bar
	 */
	void updateProgression();

	void refreshNbControllersProperty(int32 value) throw (CannotSetValue);
	void refreshActiveControllersProperty(int32 value) throw (CannotSetValue);

private:

	int32 m_registeredControllers;

	std::vector<ControllerPtr<npp::CAENPhaController> > m_phaControllers;


};

}
#endif //PHATUNINGCONTROLLER_H
