/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENCFD2DRIVER_H
#define CAENCFD2DRIVER_H

#include <Driver.h>
#include "drivers/global/Vme.h"

namespace caen_cfd2 {

/*!
 * \class CAENPhaDriver
 * \brief Main class for the CAENPhaDriver device driver
 *
 * \par
 * Class for caen_cfdDriver driver. Implements DeviceDriver methods.
 * \par
 * The CAENPhaDriver device manages the global functionality of the CAEN Pha VME Digitiser.
 */

class CAENCfd2Driver : public driver::Vme {

public:

	//! Driver type value
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 * \param[in] name the name of the device driver
	 */
	CAENCfd2Driver(const std::string& name);

	/*!
	 * \brief Destructor
	 */
	virtual ~CAENCfd2Driver();

	/*!
	 * \brief Method called for executing a command
	 *
	 * \param[in] command the command to apply on the controller
	 */
	virtual void execute(const std::string& aCommand);

	// Global properties
	Property<int32> nbChannels;
	Property<int32> nbBits;
    Property<int32> model;
    Property<int32>	channels;
    Property<int32> serialNumber;
    Property<std::string> modelName;
    Property<std::string> rocFirmware;
    Property<std::string> amcFirmware;

    // Board properties
    Property<int32> linkType;
    Property<int32> acqMode;
    Property<int32> dppAcqMode;
    Property<std::string> dppAcqModeStr;
    Property<int32> saveMode;
    Property<std::string> saveModeStr;
    Property<int32> recordLength;					// Nb samples for oscilloscope mode
    Property<int32> cfdTrap;
    Property<int32> ioLevel;
    Property<int32> triggerMode;
    Property<int32> channelMask;

	// Coincidence properties
	Property<int32> enableCoincidence;
	Property<int32> coincidenceWindow;
	Property<int32> coincidenceMask;

    Property<int32> usePn1Coincidence;

	// Channels properties in dppParams
	DynamicProperty<int32> signalDecayTime;			// M - range 0:65535
	DynamicProperty<int32> trapezoidFlatTop;		// m - range 0:1023
	DynamicProperty<int32> trapezoidRiseTime;		// k - range 0:1023
	DynamicProperty<int32> flatTopDelay;			// ftd (peaking)
	DynamicProperty<int32> triggerSmoothing;		// a - range 0:63
	DynamicProperty<int32> signalRiseTime;			// b - range 0:255
	DynamicProperty<int32> triggerThreshold;		// thr - range 0:16383
	DynamicProperty<int32> baselineMean;			// nsbl - range 0:6 (no baseline, 16, 64, 256, 1024, 4096, 16384)
	DynamicProperty<int32> peakMean;				// nspk - range 0:3 (1, 4, 16, 64 samples)
	DynamicProperty<int32> peakHoldOff;				// pkho - range 0:255
	DynamicProperty<int32> baselineHoldOff;			// blho - range 0:255
	DynamicProperty<int32> triggerHoldOff;			// trgho - range 0:63
	DynamicProperty<int32> triggerWindowSize;		// twwdt
	DynamicProperty<int32> triggerWindowEnable;		// trgwin - range 0:1
	DynamicProperty<int32> digitalGain;				// dgain - range 0:3 (1, 2, 4, 8)
	DynamicProperty<float64> trapezoidGain; 		// enf
	DynamicProperty<int32> decimation;				// decimation - range 0:3 (disabled, 2, 4 8)

	// Channels properties outside dppParam
	DynamicProperty<int32> channelActive;
	DynamicProperty<std::string> channelActiveStr;
	DynamicProperty<int32> dynamicRange;			// dynamic range 0=2V 1=0.5V - range 0:1
	DynamicProperty<std::string> dynamicRangeStr;
	DynamicProperty<int32> dcOffset;				// range 0 - 100 of full scale
	DynamicProperty<int32> preTrigger;				// range 0:1023 (nSamples = pre_trig x 2)
	DynamicProperty<int32> pulsePolarity;			// invert input - range 0:1
	DynamicProperty<std::string> pulsePolarityStr;
	DynamicProperty<int32> useFirstDerivative;		// use first derivative - range 0:1

	// Channels properties for CFD mode
	DynamicProperty<int32> cfdActive;
	DynamicProperty<std::string> cfdActiveStr;
	DynamicProperty<int32> cfdFraction;				// 00= 25% - 01=50% - 11 = 100%
	DynamicProperty<std::string> cfdFractionStr;	// 00= 25% - 01=50% - 11 = 100%
	DynamicProperty<int32> cfdDelay;				// In step of 2ns
	DynamicProperty<int32> cfdSignalZero;			// In ADC channels

	// Channels adc temperature
	DynamicProperty<int32> adcTemperature;

	// Commands
	static const std::string WRITE_PARAMETERS_COMMAND;
	static const std::string SET_MODE_COMMAND;
	static const std::string RESET_COMMAND;
	static const std::string CLEAR_COMMAND;

private:

	void refreshNbChannelsProperty(int32 aValue) throw (CannotSetValue);
	void refreshDppAcqModeProperty(int32 aValue) throw (CannotSetValue);
	void refreshSaveModeProperty(int32 aValue) throw (CannotSetValue);
	void refreshChannelActiveProperty(int32 index, int32 aValue) throw (CannotSetValue);
	void refreshPulsePolarityProperty(int32 index, int32 aValue) throw (CannotSetValue);
	void refreshDynamicRangeProperty(int32 index, int32 aValue) throw (CannotSetValue);
};

}
#endif //CAENCFD2DRIVER_H
