/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CAENQdcDef.h"
#include "CAENQdcState.h"
#include "CAENQdcDriver.h"
#include "PerfectCAENQdcDriver.h"
#include "RealCAENQdcDriver.h"
#include "drivers/global/DriversCommands.h"

namespace caen_qdc {

const std::string CAENQdcDriver::TYPE = "caen_qdc";

const std::string CAENQdcDriver::WRITE_PARAMETERS_COMMAND = "writeParam";
const std::string CAENQdcDriver::SET_MODE_COMMAND = "setAcqMode";
const std::string CAENQdcDriver::RESET_COMMAND = "reset";
const std::string CAENQdcDriver::CLEAR_COMMAND = "clear";

/*
 * Constructor
 */
CAENQdcDriver::CAENQdcDriver(const std::string& name) {

	driver::Vme::init(name);

	registerStates(new RealCAENQdcDriver(this), new PerfectCAENQdcDriver(this), new PerfectCAENQdcDriver(this));

	// Global properties
	nbChannels.init(this, SAVE, "nb_channels");
	nbBits.init(this, SAVE, "nb_bits");
    model.init(this, NOSAVE, "model");
    channels.init(this, NOSAVE, "channels");
    serialNumber.init(this, NOSAVE, "serial_number");
    modelName.init(this, NOSAVE, "model_name");
    rocFirmware.init(this, NOSAVE, "roc_firmware");
    amcFirmware.init(this, NOSAVE, "amc_firmware");

    // Board properties
    linkType.init(this, SAVE, "link_type");
    acqMode.init(this, SAVE, "acq_mode");
    dppAcqMode.init(this, SAVE, "dpp_acq_mode");
    dppAcqModeStr.init(this, NOSAVE, "dpp_acq_mode_str");
    saveMode.init(this, SAVE, "save_mode");
    saveModeStr.init(this, NOSAVE, "save_mode_str");
    recordLength.init(this, SAVE, "record_length");
    ioLevel.init(this, SAVE, "io_level");
    triggerMode.init(this, SAVE, "trigger_mode");
    chargeSensitivity.init(this, SAVE, "charge_sensitivity");
    inputSmoothing.init(this, SAVE, "input_smoothing");
	pulsePolarity.init(this, SAVE, "pulse_polarity");
	pulsePolarityStr.init(this, SAVE, "pulse_polarity_str");
    baselineMean.init(this, SAVE, "baseline_mean");
    triggerHysteresis.init(this, SAVE, "trigger_hysteresis");
    selfTrigger.init(this, SAVE, "self_trigger");
    chargePedestal.init(this, SAVE, "charge_pedestal");
    testPulseEnable.init(this, SAVE, "test_pulse_ena");
    testPulseRate.init(this, SAVE, "test_pulse_rate");
    fixedBaseline.init(this, SAVE, "fixed_baseline");

	// Group properties
    gateWidth.init(this, SAVE, "gate_width");
    preGate.init(this, SAVE, "pre_gate");
	preTrigger.init(this, SAVE, "pre_trigger");
    triggerHoldOff.init(this, SAVE, "trigger_holdoff");
	dcOffset.init(this, SAVE, "dc_offset");
	channelEnableMask.init(this, SAVE, "trigger_holdoff");

	// Channel properties
	triggerThreshold.init(this, SAVE, "trigger_threshold");
    channelActive.init(this, SAVE, "channel_active");
    channelActiveStr.init(this, NOSAVE, "channel_active_str");

	// Init the device command list
	initCommand(driver::INIT_COMMAND);
	initCommand(driver::READ_INFOS_COMMAND);
	initCommand(WRITE_PARAMETERS_COMMAND);
	initCommand(SET_MODE_COMMAND);
	initCommand(RESET_COMMAND);
	initCommand(CLEAR_COMMAND);

	// Init functions
	registerFunction(NONE_FUNCTION);

	addressType.update(driver::Vme::A32_ACCESS);
	deviceType = LEAF_DEVICE_TYPE_DEVICE_CONTAINER;

	registerRefresher(nbChannels, &CAENQdcDriver::refreshNbChannelsProperty, this);
	registerRefresher(dppAcqMode, &CAENQdcDriver::refreshDppAcqModeProperty, this);
	registerRefresher(saveMode, &CAENQdcDriver::refreshSaveModeProperty, this);
	registerRefresher(channelActive, &CAENQdcDriver::refreshChannelActiveProperty, this);
	registerRefresher(pulsePolarity, &CAENQdcDriver::refreshPulsePolarityProperty, this);
}

/*
 * Destructor
 */
CAENQdcDriver::~CAENQdcDriver() {

}

/*
 * execute
 */
void CAENQdcDriver::execute(const std::string& aCommand) {

	if ((aCommand != driver::STOP_COMMAND) && (aCommand != driver::STATUS_COMMAND)
			&& (aCommand != driver::READ_COMMAND)) {
		commandProgression = PROGRESSION_UNKNOWNSTATE_DEVICE_CONTAINER;
	}

	CAENQdcState* currentState = dynamic_cast<CAENQdcState*>(getCurrentState());
	// Check command
	if (aCommand == driver::INIT_COMMAND) {
		// Init command
		currentState->init();
	} else if (aCommand == WRITE_PARAMETERS_COMMAND) {
		// writeParam command
		currentState->writeParam();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == SET_MODE_COMMAND) {
		// clear command
		currentState->setAcqMode();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == CLEAR_COMMAND) {
		// clear command
		currentState->clear();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == RESET_COMMAND) {
		// clear command
		currentState->reset();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else if (aCommand == driver::READ_INFOS_COMMAND) {
		// Info command
		currentState->readInfos();
		commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	} else {
		//Error bad command
		//Todo , normaly it will be detect it device action driver
	}
}

void CAENQdcDriver::refreshNbChannelsProperty(int32 aValue)  throw (CannotSetValue){

	// Group properties
	channelEnableMask.resize(aValue / 8);
	triggerHoldOff.resize(aValue / 8);
	gateWidth.resize(aValue / 8);
	preGate.resize(aValue / 8);
    dcOffset.resize(aValue / 8);
	preTrigger.resize(aValue / 8);

	// Channel properties
    triggerThreshold.resize(aValue);
	channelActive.resize(aValue);
	channelActiveStr.resize(aValue);
}

void CAENQdcDriver::refreshDppAcqModeProperty(int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		dppAcqModeStr = "Oscilloscope";
		break;
	case 1:
		dppAcqModeStr = "Histogram";
		break;
	}
}

void CAENQdcDriver::refreshSaveModeProperty(int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		saveModeStr = "Energy";
		break;
	case 1:
		saveModeStr = "Time";
		break;
	case 2:
		saveModeStr = "Energy&Time";
		break;
	}
}

void CAENQdcDriver::refreshChannelActiveProperty(int32 index, int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		channelActiveStr.set(index, "Inactive");
		break;
	case 1:
		channelActiveStr.set(index, "Active");
		break;
	}
}

void CAENQdcDriver::refreshPulsePolarityProperty(int32 aValue) throw (CannotSetValue) {

	switch (aValue) {
	case 0:
		pulsePolarityStr.set("Positive");
		break;
	case 1:
		pulsePolarityStr.set("Negative");
		break;
	}
}

}
