/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealCAENSy2527Driver.h"

#include <controllers/common/detector_hv/DetectorHVCommon.h>
#include "CAENSy2527Driver.h"

using namespace std;

namespace caen_sy2527 {

/*
 * Constructor
 */
RealCAENSy2527Driver::RealCAENSy2527Driver(CAENSy2527Driver* owner) :
	CAENSy2527State(owner) {

}

/*
 * Destructor
 */
RealCAENSy2527Driver::~RealCAENSy2527Driver() {

}

/*!
 * \brief Init command implementation
 */
void RealCAENSy2527Driver::init() {

	int32 link = LINKTYPE_TCPIP;
	//int32 sysType = 1;				// Crate Sy2527
	int32 sysType = 3;				// Crate Sy5527
	CAENHVRESULT  ret;

	ret = CAENHV_InitSystem((CAENHV_SYSTEM_TYPE_t)sysType, link, (void *)owner()->ipAddress().c_str(), "admin", "admin", &owner()->m_crateHandle);

	if (ret == CAENHV_OK) {
		readInfos();
	} else {
		cout << "Connection error" << endl;
	}
}

/*!
 * \brief Read Infos command implementation
 */
void RealCAENSy2527Driver::readInfos() {

	CAENHVRESULT  ret;

	unsigned short	nrOfSlots = 0;
	unsigned short* SerNumList;
	unsigned short* nrOfChannelList;
	unsigned char	*FmwRelMinList;
	unsigned char	*FmwRelMaxList;
	char*			ModelList;
	char*			DescriptionList;

    ret = CAENHV_GetCrateMap(owner()->m_crateHandle, &nrOfSlots, &nrOfChannelList, &ModelList, &DescriptionList, &SerNumList, &FmwRelMinList, &FmwRelMaxList );

	if (ret == CAENHV_OK) {
		owner()->nbSlots = nrOfSlots;
		owner()->channelsPerBoard.resize(nrOfSlots);
		owner()->boardModel.resize(nrOfSlots);
		owner()->boardDescription.resize(nrOfSlots);
		owner()->boardFirmware.resize(nrOfSlots);
		owner()->boardSerial.resize(nrOfSlots);

		char* m = ModelList;
		char* d = DescriptionList;
		char  firmware[10];

		for(int32 i = 0; i < nrOfSlots; i++ , m += strlen(m) + 1, d += strlen(d) + 1 ) {
			if( *m == '\0' ) {
				owner()->boardModel.set(i,"None");
				owner()->boardDescription.set(i,"None");
				owner()->boardFirmware.set(i,"None");
			} else {
				owner()->boardModel.set(i,m);
				owner()->boardDescription.set(i,d);
				sprintf(firmware,"%d.%d",FmwRelMaxList[i],FmwRelMinList[i]);
				owner()->boardFirmware.set(i,(string)firmware);
			}
			owner()->boardSerial.set(i,(int32)SerNumList[i]);
			owner()->channelsPerBoard.set(i,nrOfChannelList[i]);
		}

		free(SerNumList), free(ModelList), free(DescriptionList), free(FmwRelMinList), free(FmwRelMaxList), free(nrOfChannelList);
	} else {
		cout << "Error reading crate info" << endl;
	}
}

/*
 * read
 */
void RealCAENSy2527Driver::read(CAENSy2527BoardDriver* aBoard) {

	readParam(aBoard);
}

/*
 * readParam
 */
void RealCAENSy2527Driver::readParam(CAENSy2527BoardDriver* aBoard) {

	CAENHVRESULT ret;

	int32 board = aBoard->getChannel() - 1;
	int32 channels = owner()->channelsPerBoard.get(board);

	unsigned short chList[channels];
	float		   fParValList[channels];

	if(channels > 0){

		for (int32 i=0; i<channels; ++i) {
			chList[i] = (unsigned short) i;
		}

		ret = CAENHV_GetChParam(owner()->m_crateHandle, board, "VMon", channels, chList, fParValList);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i ) {
				aBoard->vMon.set(i, fParValList[i]);
			}
		} else {
			cout << "Error reading board info" << endl;
		}

		ret = CAENHV_GetChParam(owner()->m_crateHandle, board, "IMon", channels, chList, fParValList);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i ) {
				aBoard->iMon.set(i, fParValList[i]);
			}
		} else {
			cout << "Error reading board info" << endl;
		}
	}
}

/*
 * writeParam
 */
void RealCAENSy2527Driver::writeParam(CAENSy2527BoardDriver* aBoard) {

	CAENHVRESULT ret;

	int32 board = aBoard->getChannel() - 1;
	int32 channels = owner()->channelsPerBoard.get(board);

	if(channels > 0) {
		for (int32 i=0; i<channels; ++i) {
			unsigned short channel = i;
			const char* channelName = aBoard->channelName(i).c_str();
			float aValue = aBoard->v0Set(i);
			ret = CAENHV_SetChName(owner()->m_crateHandle, board, 1, &channel, channelName);
			ret += CAENHV_SetChParam(owner()->m_crateHandle, board, "V0Set", 1, &channel, &aValue);
			aValue = aBoard->i0Set(i);
			ret += CAENHV_SetChParam(owner()->m_crateHandle, board, "I0Set", 1, &channel, &aValue);
			aValue = aBoard->rampUp(i);
			ret += CAENHV_SetChParam(owner()->m_crateHandle, board, "RUp", 1, &channel, &aValue);
			aValue = aBoard->rampDown(i);
			ret += CAENHV_SetChParam(owner()->m_crateHandle, board, "RDWn", 1, &channel, &aValue);
			if (ret != CAENHV_OK) {
				cout << "Error writing board parameters" << endl;
				break;
			}
		}
	}
}

/*
 * start
 */
void RealCAENSy2527Driver::start(CAENSy2527BoardDriver* aBoard) {

	CAENHVRESULT ret;

	int32 board = aBoard->getChannel() - 1;
	int32 channels = owner()->channelsPerBoard.get(board);

	if(channels > 0) {

		writeParam(aBoard);

		unsigned long aValue = -1;
		for (int32 i=0; i<channels; ++i) {
			unsigned short channel = i;
			if (aBoard->power(i) == "On") {
				aValue = 1;
			} else if (aBoard->power(i) == "Off") {
				aValue = 0;
			}
			ret = CAENHV_SetChParam(owner()->m_crateHandle, board, "Pw", 1, &channel, &aValue);
			if (ret != CAENHV_OK) {
				cout << "Error switching on channels" << endl;
				break;
			}
		}
	}
}

/*
 * stop
 */
void RealCAENSy2527Driver::stop(CAENSy2527BoardDriver* aBoard) {

	CAENHVRESULT ret;

	int32 board = aBoard->getChannel() - 1;
	int32 channels = owner()->channelsPerBoard.get(board);

	unsigned short chList[channels];

	if(channels > 0) {

		for (int32 i=0; i<channels; ++i) {
			chList[i] = (unsigned short) i;
		}

		for (int32 i=0; i<channels; ++i) {
			if (aBoard->channelStatus.get(i) != "On") {
				float64 aValue = aBoard->vMon(i);
				ret = CAENHV_SetChParam(owner()->m_crateHandle, board, "V0Set", channels, chList, &aValue);
				if (ret != CAENHV_OK) {
					cout << "Error stopping HV" << endl;
					break;
				}
			}
		}
	}
}

/*
 * readStatus
 */
void RealCAENSy2527Driver::readStatus(CAENSy2527BoardDriver* aBoard) {

	CAENHVRESULT ret;

	int32 board = aBoard->getChannel() - 1;
	int32 channels = owner()->channelsPerBoard.get(board);

	unsigned short chList[channels];
	int32 		   iParValList[channels];

	if(channels > 0) {

		int32 localStatus = 0;

		for (int32 i=0; i<channels; ++i) {
			chList[i] = (unsigned short) i;
		}

		ret = CAENHV_GetChParam(owner()->m_crateHandle, board, "Status", channels, chList, iParValList);

		if( ret == CAENHV_OK) {
			for (int32 i=0; i<channels; ++i) {
				string str_status="";
				for(int j=9; j>=0; j--) {
					if(iParValList[i] & (1 << j)) {
						str_status = STATUSCHANNEL[j];
						localStatus += j;
						break;
					}
				}
				aBoard->channelStatus.set(i, str_status);
			}
		} else {
			cout << "Error reading channel status" << endl;
		}

		aBoard->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}
}

/*
 * powerOn
 */
void RealCAENSy2527Driver::powerOn(CAENSy2527BoardDriver* aBoard) {

}

/*
 * powerOff
 */
void RealCAENSy2527Driver::powerOff(CAENSy2527BoardDriver* aBoard) {

}

}
